/*
 * Copyright (C) 2009 SENJU, Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* for some old systems */
#define _REENTRANT
#define POSIX_C_SOURCE 199506L
#include <pthread.h>

#include <assert.h>
#include <err.h>
#include <stdio.h>
#include <unistd.h>

#include "lib.h"

void *func(void *arg)
{
	int err, lsock, sock;
	ssize_t ssz;
	char a[BUFSIZ], *p;
	struct sockaddr_in addr;

	lsock = (int)arg;
	while (1) {
		socklen_t l = sizeof(addr);

		sock = accept(lsock, (struct sockaddr *)&addr, &l);
		assert(sock >= 0);

		ssz = read(sock, a, sizeof(a));
		assert(ssz > 0);
		assert(ssz != sizeof(a));

		p = get_path(a);
		if (p)
			do_sendfile(sock, p, /*delay*/0);
		err = close(sock);
		assert(!err);
	}

	return NULL;
}

int main(int argc, char *argv[])
{
	const int nthread = num_cpu();
	int err, port, lsock, i;
	pthread_t th[nthread];

	assert(nthread >= 1);

	/* port = 80; */
	port = IPPORT_USERRESERVED;
	warnx("listening on port %d", port);
	lsock = simple_server_socket(port, 64, /*non-blocking*/0);
	assert(lsock >= 0);

	warnx("creating %d threads", nthread);
	for (i = 0; i < nthread; i++) {
		err = pthread_create(th + i, NULL, func, (void *)lsock);
		assert(!err);
	}

	for (i = 0; i < nthread; i++)
		err = pthread_join(th[i], NULL);

	return 0;
}
