/*
 * Copyright (C) 2009 SENJU, Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/epoll.h>
#include <assert.h>
#include <err.h>
#include <fcntl.h>

#include "lib.h"
#include "sample_sub.h"

int do_accept(int lsock, int epfd)
{
	int err, sock;
	struct sockaddr_in addr;
	struct epoll_event ev;
	socklen_t l = sizeof(addr);

	sock = accept(lsock, (struct sockaddr *)&addr, &l);
	assert(sock >= 0);

	err = set_nonblock(sock, 1);
	assert(!err);

	ev.events = EPOLLIN | EPOLLHUP | EPOLLET;
	ev.data.fd = sock;
	err = epoll_ctl(epfd, EPOLL_CTL_ADD, sock, &ev);
	assert(!err);

	return err;
}

int main(int argc, char *argv[])
{
	const int maxopen = num_fd();
	int err, port, lsock, epfd, nfd, i, sock;
	struct epoll_event ev;

	assert(maxopen > 0);
	err = sample_init(maxopen);
	assert(!err);

	/* port = 80; */
	port = IPPORT_USERRESERVED;
	warnx("listening on port %d", port);
	lsock = simple_server_socket(port, 64, /*non-blocking*/1);
	assert(lsock >= 0);

	epfd = epoll_create(1);
	assert(!epfd >= 0);
	ev.events = EPOLLIN | EPOLLHUP /* | EPOLLET */;
	ev.data.fd = lsock;
	err = epoll_ctl(epfd, EPOLL_CTL_ADD, lsock, &ev);
	assert(!err);

	while (1) {
		struct epoll_event e[maxopen];

		nfd = epoll_wait(epfd, e, maxopen, 10 * 1000);
		assert(nfd > 0);

		for (i = 0; i < nfd; i++) {
			sock = e[i].data.fd;
			if (sock == lsock)
				do_accept(sock, epfd);
			else if (e[i].events & EPOLLIN)
				do_read(sock, epfd);
			else if (e[i].events & EPOLLOUT)
				do_sfc(sock, epfd);
			else if (e[i].events & EPOLLHUP)
				do_close(sock, epfd);
		}
	}
	return 0;
}
