/*
 * Copyright (C) 2009 SENJU, Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/epoll.h>
#include <assert.h>
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "lib.h"
#include "sample_sub.h"

int do_sock(int pfd, int epfd)
{
	int sock, err;
	struct epoll_event ev;

	sock = recv_sock(pfd);
	assert(sock >= 0);

	err = set_nonblock(sock, 1);
	assert(!err);

	ev.events = EPOLLIN | EPOLLHUP | EPOLLET;
	ev.data.fd = sock;
	err = epoll_ctl(epfd, EPOLL_CTL_ADD, sock, &ev);
	assert(!err);

	return sock;
}

void child(int pfd)
{
	const int maxopen = num_fd();
	int err, nfd, i, sock, epfd;
	struct epoll_event ev;

	assert(maxopen > 0);
	err = sample_init(maxopen);
	assert(!err);

	epfd = epoll_create(1);
	assert(!epfd >= 0);
	ev.events = EPOLLIN | EPOLLHUP /* | EPOLLET */;
	ev.data.fd = pfd;
	err = epoll_ctl(epfd, EPOLL_CTL_ADD, pfd, &ev);
	assert(!err);

	while (1) {
		struct epoll_event e[maxopen];

		nfd = epoll_wait(epfd, e, maxopen, 10 * 1000);
		assert(nfd > 0);

		for (i = 0; i < nfd; i++) {
			sock = e[i].data.fd;
			if (sock == pfd)
				do_sock(sock, epfd);
			else if (e[i].events & EPOLLIN)
				do_read(sock, epfd);
			else if (e[i].events & EPOLLOUT)
				do_sfc(sock, epfd);
			else if (e[i].events & EPOLLHUP)
				do_close(sock, epfd);
		}
	}
}

int main(int argc, char *argv[])
{
	int nchild = num_cpu() - 1;
	int err, port, lsock, i, fd[nchild], n, sock;
	struct sockaddr_in addr;
	pid_t pid;

	if (nchild <= 0) {
		warnx("This sample does not suit on a single processor");
#if 1
		sleep(1);
		nchild = 1;
#else
		return ENOSYS;
#endif
	}

	warnx("creating %d child processes", nchild);
	for (i = 0; i < nchild; i++) {
		int fds[2];

		err = socketpair(AF_LOCAL, SOCK_DGRAM, 0, fds);
		assert(!err);
		fd[i] = fds[0];

		pid = fork();
		assert(pid != -1);
		if (!pid) {
			/* child */
			int j;

			for (j = 0; j < i; j++)
				close(fd[j]);
			close(fds[0]);
			child(fds[1]);
		}
		close(fds[1]);
	}

	/* port = 80; */
	port = IPPORT_USERRESERVED;
	warnx("listening on port %d", port);
	lsock = simple_server_socket(port, 64, /*non-blocking*/0);
	assert(lsock >= 0);

	n = 0;
	while (1) {
		socklen_t l = sizeof(addr);

		sock = accept(lsock, (struct sockaddr *)&addr, &l);
		assert(sock >= 0);

		err = send_sock(fd[n++], sock);
		assert(!err);
		n %= nchild;
		close(sock);
	}

	return 0;
}
