/*
 * Copyright (C) 2009 SENJU, Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/epoll.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "lib.h"
#include "sample_sub.h"

static char **path;

int do_read(int sock, int epfd)
{
	int err;
	ssize_t ssz;
	char a[BUFSIZ];
	struct epoll_event ev;

	ssz = read(sock, a, sizeof(a));
	assert(ssz > 0);
	assert(ssz != sizeof(a));

	path[sock] = get_path(a);

	ev.events = EPOLLOUT | EPOLLHUP | EPOLLET;
	ev.data.fd = sock;
	err = epoll_ctl(epfd, EPOLL_CTL_MOD, sock, &ev);
	assert(!err);

	return err;
}

int do_close(int sock, int epfd)
{
	int err;

	err = epoll_ctl(epfd, EPOLL_CTL_DEL, sock, NULL);
	assert(!err);
	err = close(sock);
	assert(!err);

	return err;
}

/* sendfile and close */
int do_sfc(int sock, int epfd)
{
	if (path[sock])
		do_sendfile(sock, path[sock], /*delay*/0);

	return do_close(sock, epfd);
}

int sample_init(int maxopen)
{
	path = calloc(sizeof(*path), maxopen);
	return path ? 0 : -1;
}

