/*
 * Copyright (C) 2009 SENJU, Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/sendfile.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <strings.h>
#include <time.h>
#include <unistd.h>

#include "http.h"

/* extract a file path from simple HTTP request */
char *get_path(char *p)
{
	char *q;

	while (isspace(*p))
		p++;
	if (!strncasecmp(p, "GET", 3)) {
		p += sizeof("GET");
		while (isspace(*p))
			p++;
		if (*p == '/') {
			q = p + 1;
			while (!isspace(*q))
				q++;
			*q = 0;
			return p; /* success */
		}
	}
	return NULL;
}

static int sleep_well(struct timespec *ts)
{
	int err;
	struct timespec tr;

	while (1) {
		/* no signal will be delivered */
		err = nanosleep(ts, &tr);
		if (!err || errno != EINTR)
			break;
		*ts = tr;
	}

	return err;
}

/*
 * reply to the HTTP request with a simple HTTP response header
 * and filedata
 */
int do_sendfile(int sock, char *path, unsigned int delay_ms)
{
	int err, fd, v, rerr;
	struct stat st;
	ssize_t ssz;

#define HttpRes \
	"HTTP/1.0 200 OK\r\n" \
	"Last-modified: Thu, 1 Jan 1970 00:00:00 GMT\r\n" \
	"\r\n"

	/* puts(path); */
	if (delay_ms >= 1 && delay_ms < 1000) {
		struct timespec ts = {
			.tv_sec		= 0,
			.tv_nsec	= delay_ms * 1000 * 1000
		};

		err = sleep_well(&ts);
		if (err)
			goto out;
	}

	fd = open(path, O_RDONLY);
	err = fd;
	if (fd < 0)
		goto out;

	err = fstat(fd, &st);
	if (err)
		goto out_fd;

	v = 1;
	err = setsockopt(sock, IPPROTO_TCP, TCP_CORK, &v, sizeof(v));
	if (err)
		goto out_fd;

	ssz = write(sock, HttpRes, sizeof(HttpRes) - 1);
	if (ssz != sizeof(HttpRes) - 1)
		goto out_fd;

	ssz = sendfile(sock, fd, NULL, st.st_size);
	if (ssz != st.st_size)
		goto out_fd;

	v = 0;
	err = setsockopt(sock, IPPROTO_TCP, TCP_CORK, &v, sizeof(v));

 out_fd:
	rerr = close(fd);
	if (!err)
		err = rerr;
 out:
	return err;
}
